/*************************************************************
 * ADG412を使用したセレクター用　LCD表示器
 * 使用PIC16F1938
 * 使用コンパイラ　ＸＣ８ Ver 1.45
 * 2021年3月01日 
 *************************************************************/
#include    <xc.h>
#include    <stdio.h>
#include    "lcd_lib.h"

/*****コンフィグレーション設定*********/
// コンフィギュレーション１の設定
#pragma config FOSC = INTOSC    // 内部ｸﾛｯｸを使用する(INTOSC)
#pragma config WDTE = OFF       // ｳｵｯﾁﾄﾞｯｸﾞﾀｲﾏｰ無し(OFF)
#pragma config PWRTE = ON       // 電源ONから64ms後にﾌﾟﾛｸﾞﾗﾑを開始する(ON)
#pragma config MCLRE = ON      // 外部ﾘｾｯﾄ信号を使用
#pragma config CP = OFF         // ﾌﾟﾛｸﾞﾗﾑﾒﾓﾘｰを保護しない(OFF)
#pragma config CPD = OFF        // ﾃﾞｰﾀﾒﾓﾘｰを保護しない(OFF)
#pragma config BOREN = ON       // 電源電圧降下常時監視機能ON(ON)
#pragma config CLKOUTEN = OFF   // CLKOUTﾋﾟﾝをRA6ﾋﾟﾝで使用する(OFF)
#pragma config IESO = OFF       // 外部・内部ｸﾛｯｸの切替えでの起動はなし(OFF)
#pragma config FCMEN = OFF      // 外部ｸﾛｯｸ監視しない(FCMEN_OFF)

// コンフィギュレーション２の設定
#pragma config WRT = OFF        // Flashﾒﾓﾘｰを保護しない(OFF)
#pragma config PLLEN = OFF      // 動作クロックを32MHzでは動作させない(OFF)
#pragma config STVREN = ON      // スタックがオーバフローやアンダーフローしたらリセットをする(ON)
#pragma config BORV = HI        // 電源電圧降下常時監視電圧(2.5V)設定(HI)
#pragma config LVP = OFF        // 低電圧プログラミング機能使用しない(OFF)

/** 内部クロック　８ＭＨｚ **/
#define  _XTAL_FREQ     8000000	//DELAY 用

/********　変数定義　************/
#define	LINE1			0x80    // １行目設定
#define	LINE2			0xc0    // ２行目設定

#define UARTPRINT       0       //シリアル出力に設定
#define LCDPRINT        1       //ＬＣＤ表示に設定

#define ON              1       //
#define OFF             0       //
#define DELAY           97          //赤外線ウエイト調整用
		
unsigned	int     i;
unsigned	int     eepdata;        //EEPROMデータ
unsigned    int     p_mode;         //出力先切り替え
unsigned    int     s_mode;         //入力モード　シリアル・赤外線切り替え

//赤外線用
static      int     data[36];       //赤外線のHi・low時間の保管用
static      int     manudata;       //PIC内部でのメーカー番号
static  	int     manucord2;      //リモコンのメーカー識別用
static      int     manu[10] = {0x10,0x18,0x30,0x40,0x50,0x15,0xaa,0x80,0x04,0xf1}; //メーカーコード
static      int     LR_c[10] = {0x20,0x0a,0x05,0x0f,0x09,0x00,0x13,0x12,0x0b,0x00};	//入力切替
static      int     select_no;		//input no
static      int     inputdata;  	//受信信号からデーターへの変換用
static      int     manucord;		//リモコンのメーカー識別用
static      int     Received_buf;	//1回のコマンドを一時保管、リピート用
static      long	starttime = 0;	//起動後の5秒間カウント用

/******** サブ関数   **************/
 void putch(unsigned char byte);

void manuset(void);
void infread(void);
void select(void);
/********  メイン開始  ************/
void main(void) {
    /**  初期設定 **/
    OSCCON = 0x72;              //　内臓クロック８HMz指定(BPLL Off)
    /**ポートの設定  ***/
    ANSELA = 0x00;              //A/D停止
    ANSELB = 0x00;              //A/D停止
    TRISA = 0b00000000;         //A0?A5端子　出力設定
    TRISB = 0b00000000;         //LCD用
    TRISC = 0b10000001;         //シリアルTX(C6)・RX(C7)設定　赤外線入力(C0)  
  
    /**　シリアル通信の設定 **/
    TXSTA = 0x24;				//シリアル受信設定
    RCSTA = 0x90;				//シリアル送信設定
	BAUDCON=0x08;				//
	SPBRG = 103;				//ボーレート  19.2kbps
    
	PEIE = 1;					//基本モジュール　タイマー０・外部割り込み以外の割り込み許可
	RCIE = 1;					//受信割り込み許可
	GIE = 0;					//すべての割り込み禁止
	RCIF = 0;					//割り込みフラグをクリア

    PORTA= 0b00110000;			//ポートＡすべてLOW
     
 /************　ＬＣＤモジュール 表示テストプログラム　********************/
    lcd_init();                 //　LCD初期化
    lcd_cmd(0x01);              //　表示クリア
    lcd_cmd(0x0c);              //　ブリンク非表示
 
 //   p_mode = UARTPRINT;         //　出力先をシリアル通信にする
 //   printf("TEST Ver 1.c \r\n");//  テキスト表示
    
    p_mode = LCDPRINT;           //　出力先をLCD表示にする  
	lcd_cmd(LINE1);				//表示する行をクリアする
	printf("  ADG412B Select ");
	lcd_cmd(LINE2);				//表示する行をクリアする
	printf("   Ver. 1.2      ");

  	__delay_ms(1000);			//500ms程度だと起動時にlowと判定される。

//過去のセレクト番号をＥＥＰＲＯＭから読み込み
	select_no = eeprom_read(0x01);
    
    p_mode = UARTPRINT;         //　出力先をシリアル通信にする
    printf("select_no = %d \r\n",select_no);      // 
    
	if	(select_no > 0x04)      //セレクタ番号が４以上なら１にする
	{
		eeprom_write(0x01,0x01);
		select_no = 0x01;
	}
    p_mode = UARTPRINT;         //　出力先をシリアル通信にする
    printf("select_no2 = %d \r\n",select_no);   //セレクト番号を表示する。

//=============================================================	
//    電源投入後　5秒間に今後使用したいリモコンを操作する
//    （ボタンはどれでもいい）	
// 	　使用するのリモコンのメーカーをセット
//=============================================================
	manudata = eeprom_read(0x02);   //前回のメーカー番号をEEPROMから読み込む

	//　EEPROMのデータが0xffの場合　秋月電子製をセットする。
	if	(manudata == 0xff)
	{
		eeprom_write(0x02,0x00);
		manudata = 0x00;
	}
	
    p_mode = UARTPRINT;         //　出力先をシリアル通信にする
    printf("manudata = %d \r\n",manudata);      // 

    p_mode = LCDPRINT;           //　出力先をLCD表示にする  
	lcd_cmd(LINE1);				//表示する行をクリアする
	printf("   MAKER SET    ");
	lcd_cmd(LINE2);				//表示する行をクリアする
	printf("     WAIT       ");    

	while(starttime < 2500)
	{
        if(RC0)    
		{
			__delay_ms(2);
			starttime++ ;
		}
		
		else					//赤外線リモコンの操作が行われた。
		{
			manuset();                  //メーカーの特定の処理を行う
            p_mode = LCDPRINT;          //　出力先をLCD表示にする  
			lcd_cmd(LINE1);				//
			printf("   Change!!!    ");
			lcd_cmd(LINE2);				//
			printf("    MAKAER  %X   ",manudata);
			
            eeprom_write(0x01,0x01);    //セレクトを１にする
            select_no = 0x01;    
            
            __delay_ms(1500);
            break;
		}
	}

    lcd_cmd(0x01);              //　表示クリア
    p_mode = LCDPRINT;           //　出力先をLCD表示にする  
	lcd_cmd(LINE1);				//
    printf("     INPUT      ");
    select();                   //入力切り替え処理
    
     while (1) 
	{
        while(RC0){}            //赤外線リモコンからの操作待ち
        infread();              //赤外線リモコンからの受信処理
        
        if (manu[manudata] == manucord)		//現在設定しているメーカーか？
		{		
			if(LR_c[manudata] == inputdata)
			{
				select_no++;
				if(select_no == 5){ select_no = 1;}
				eeprom_write(0x01,select_no);
                select();
			}					
		}
    }
}

/***************************************************
 *  1文字出力
 ***************************************************/

void putch(unsigned char data){

    //UART　出力
    if (p_mode == 0 )
    {
		RCSTAbits.SPEN=1;
		TXREG=data;
		while(!TXIF)continue;
		TXIF=0;
    }   
    //LCD　出力
    else
    {
        lcd_data(data);            
    }
}

/*=======================================================
*   メーカー識別設定
*=======================================================*/
void	manuset()
{
	int	manucord;
	lcd_cmd(LINE2);				//表示する行をクリアする
	printf("     set       ");
	
//	スタートビットの長さを測る  初回送信　9ms　  		
//	while(~(RC0)) {	}	
	while(RC0 == 0) {}	 	

//	スタートビット後のLOW時間、　初回送信　4.5ms　/　リピート　2.25ms
	while(RC0) {}
//	1回目のコマンド　だったら　データを取り込む	

	for(i = 0;i < 26  ;i++)
	{	
    	while(RC0 == 0) {}	 	
		
		while(RC0) { 
			__delay_us(DELAY);
			data[2+i]++;
			}				
		}
					
//メーカー判別
		manucord = 0;
		if (data[2]>10){manucord ++;}
		if (data[3]>10){manucord = manucord + 2;}
		if (data[4]>10){manucord = manucord + 4;}
		if (data[5]>10){manucord = manucord + 8;}
		if (data[6]>10){manucord = manucord + 16;}
		if (data[7]>10){manucord = manucord + 32;}
		if (data[8]>10){manucord = manucord + 64;}
		if (data[9]>10){manucord = manucord + 128;}
			
		switch (manucord)
		{
			case 0x10: manudata = 0;break;	//秋月電子
			case 0x18: manudata = 1;break;	//NEC
			case 0x30: manudata = 2;break;	//サンヨー
			case 0x40: manudata = 3;break;	//東芝
			case 0x50: manudata = 4;break;	//日立
			case 0x15: manudata = 5;break;	//富士通				
			case 0xAA: manudata = 6;break;	//パイオニア
			case 0x80: manudata = 7;break;	//オリオン							
			case 0x04: manudata = 8;break;	//LG
			case 0xF1: manudata = 9;break;	//ｂｙ：D:sign　（不明メーカー）
		}
		
		eeprom_write(0x02,manudata);        //メーカーをEEPROMに記録
}


/*=======================================================
*   赤外線信号受信処理
*=======================================================*/

void infread(void)
{
   manucord = 0;
   manucord2 = 0;
   inputdata = 0;
    //　１・０判別用変数を初期値に戻す。
		for (i = 0;i < 35;i++ )
		{
			data[i]=0;
		}		

//		スタートビットの長さを測る  初回送信　9ms　  		
		while(RC0 == 0) {
			__delay_us(DELAY);
			data[0]++;
		}
	
//		スタートビット後のHIGH時間、　初回送信　4.5ms　/　リピート　2.25ms
		while(RC0){
			__delay_us(DELAY);
			data[1]++;
		}
/*****************************************************
 * 赤外線リモコンのコマンドデータを解析する
 * メーカー識別用コード１６ビット
 * データコード　　　　　８ビット
 * データコードの反転部分は今回読み取っていない
******************************************************/

		if (data[0] > 80 && data[1] > 40){		//初回送信かリピートか判定
		for(i = 0;i < 26  ;i++)
		{	
			while(RC0 == 0) {}	 
//　LOWの時間を測る、
			while(RC0) { 
			__delay_us(DELAY);
			data[2+i]++;
			}				
		}	
//　コード数値化。
//　LOWの時間が1ms以上の時１とする。
//		メーカー識別コード

			if (data[2]>10){manucord ++;}
			if (data[3]>10){manucord = manucord + 2;}
			if (data[4]>10){manucord = manucord + 4;}
			if (data[5]>10){manucord = manucord + 8;}
			if (data[6]>10){manucord = manucord + 16;}
			if (data[7]>10){manucord = manucord + 32;}
			if (data[8]>10){manucord = manucord + 64;}
			if (data[9]>10){manucord = manucord + 128;}
            p_mode = UARTPRINT;         //　出力先をシリアル通信にする
			printf("manucord1  :%X \r\n",manucord);	

//		メーカー識別コード（反転）

			if (data[10]>10){manucord2 ++;}
			if (data[11]>10){manucord2 = manucord2 + 2;}
			if (data[12]>10){manucord2 = manucord2 + 4;}
			if (data[13]>10){manucord2 = manucord2 + 8;}
			if (data[14]>10){manucord2 = manucord2 + 16;}
			if (data[15]>10){manucord2 = manucord2 + 32;}
			if (data[16]>10){manucord2 = manucord2 + 64;}
			if (data[17]>10){manucord2 = manucord2 + 128;}
            p_mode = UARTPRINT;         //　出力先をシリアル通信にする
			printf("manucord2 :%X \r\n",manucord2);	

//		データコード
				
			if (data[18]>10){inputdata ++;}
			if (data[19]>10){inputdata = inputdata + 2;}
			if (data[20]>10){inputdata = inputdata + 4;}
			if (data[21]>10){inputdata = inputdata + 8;}
			if (data[22]>10){inputdata = inputdata + 16;}
			if (data[23]>10){inputdata = inputdata + 32;}
			if (data[24]>10){inputdata = inputdata + 64;}
			if (data[25]>10){inputdata = inputdata + 128;}
            
            p_mode = UARTPRINT;         //　出力先をシリアル通信にする
			printf("inputdata :%X \r\n",inputdata);	
			
			__delay_ms(100);	
		
		}

//		リピートの場合
		else if(data[0] > 80 && data[1] > 20)
		{
            p_mode = UARTPRINT;         //　出力先をシリアル通信にする
            printf("REPE \r\n");
			__delay_ms(100);

		}	     
 
}
/*=======================================================
*   入力切り替え処理
*=======================================================*/
void    select(void)
{
    PORTA= 0b00110000;	//ポートＡすべてlow
    
    p_mode = LCDPRINT;           //　出力先をLCD表示にする  
	switch (select_no)
	{
		case 0x01:	
			RA0 = ON;
			lcd_cmd(LINE2);				//表示する行をクリアする
			printf("      NO.1      ");
		break;
		
		case 0x02:	
			RA1 = ON;
			lcd_cmd(LINE2);				//表示する行をクリアする
			printf("      NO.2      ");
		break;					

		case 0x03:	
			RA2 = ON;
			lcd_cmd(LINE2);				//表示する行をクリアする
			printf("      NO.3      ");
		break;
		
		case 0x04:	
			RA3 = ON;
			lcd_cmd(LINE2);				//表示する行をクリアする
			printf("      NO.4      ");
		break;
	}  
}